using System;
using System.Collections.Generic;
using System.Text;
using System.Web.Services;
using System.Web.Services.Protocols;
using System.IO;

namespace AttachmentsRM
{
   class AttachmentsRM
   {
      static void Main(string[] args)
      {
         ttsoapcgi svr = null;
         long cookie = 0;

         try
         {
            svr = new ttsoapcgi();

            CProject[] projs = svr.getProjectList("SOAPUser", "myPassword");
            cookie = svr.ProjectLogon(projs[0], "SOAPUser", "myPassword");

            // Enable formatted text support
            svr.formattedTextSupport(cookie, true);

            // Create a requirement
            Console.WriteLine("Create a new requirement");
            CRequirement req = new CRequirement();
            req.type = "Functional Requirement";
            req.enteredBy = "User, SOAP";
            req.dateEntered = System.DateTime.Now;
            req.dateEnteredSpecified = true;
            req.summary = "Rich text fields shall display inline image attachments";

            // Add a file attachment
            CFileAttachment reqAttach = GetFileAttachmentToAdd(@"C:\Users\horakj\SOAP\Files\RichTextOverview.docx");
            req.attachmentList = new CFileAttachment[] { reqAttach };
            
            // Add an event with an attachment
            CEvent rqReviewNote = new CEvent();
            rqReviewNote.name = "Review Note";
            rqReviewNote.user = "User, SOAP";
            rqReviewNote.date = System.DateTime.Now;
            rqReviewNote.notes = "Supported image types need to be defined.";
            
            CFileAttachment rqEvtAttach = GetFileAttachmentToAdd(@"C:\Users\horakj\SOAP\Files\WebSafeImgComparison.xlsx");
            rqReviewNote.attachmentlist = new CFileAttachment[] { rqEvtAttach };
            
            req.eventList = new CEvent[] { rqReviewNote };
            
            // Add an inline attachment to the requirement description
            req.description = @"<p>The application shall support inline image attachments within rich text fields.<br /><br />
               Supported image types include:</p>
               <ul>
                  <li>.png</li>
                  <li>.bmp</li>
                  <li>.jpeg</li>
                  <li>.gif</li>
               </ul>
               <br /><p>Attachments can be chosen and scaled from the insert image dialog.<br /><br />
               <img src='C:\Users\horakj\SOAP\Files\InsertImage.png' /><br /><br />
               Attachments <em>may</em> be scaled by specifying a height and width attribute within the HTML markup.</p>";
            
            CFileAttachment rqDesc = GetFileAttachmentToAdd(@"C:\Users\horakj\SOAP\Files\InsertImage.png");
            req.descriptionInlineAttachList = new CFileAttachment[] { rqDesc };

            // Add the requirement
            Console.WriteLine("Add the requirement");
            req.recordid = svr.addRequirement(cookie, req);
            req = svr.getRequirementByRecordID(cookie, req.recordid, false);
            
            // Print the contents of the description field
            CFileAttachment inlineAttach = req.descriptionInlineAttachList[0];
            Console.WriteLine("After saving, the attachment named {0} in the description field has an archive name of {1}",
               inlineAttach.mstrFileName, inlineAttach.mstrArchiveName);
            Console.WriteLine("\nHere is the HTML within the requirement description after the save:\n\n{0}", req.description);

            // Save requirement attachment
            Console.WriteLine("\nRetrieving file attachment data for archive {0}", inlineAttach.mstrArchiveName);
            CFileAttachment atch = svr.getAttachment(cookie, "Requirement", req.recordid, 0, inlineAttach.mstrArchiveName);
            
            Console.WriteLine("File data retrieved, writing to the file system");
            WriteFileAttachment(atch);
         }
         catch (SoapException e)
         {
            Console.WriteLine("Soap fault: {0} {1}", e.Detail.InnerText, e.Message.ToString());
         }
         catch (Exception e)
         {
            Console.WriteLine(e.ToString());
         }
         finally
         {
            try
            {
               if (svr != null)
               {
                  svr.DatabaseLogoff(cookie);
                  Console.WriteLine("Logged Off");
               }
            }
            catch (SoapException e)
            {
               Console.WriteLine("Soap fault: {0} {1}", e.Detail.InnerText, e.Message.ToString());
            }
            catch (Exception e)
            {
               Console.WriteLine(e.ToString());
            }
         }
      }

      static void WriteFileAttachment(CFileAttachment atch)
      {
         FileStream outputStream = File.Create(atch.mstrFileName);
         BinaryWriter writer = new BinaryWriter(outputStream);
      
         writer.Write(atch.mpFileData);
         outputStream.Close();
         Console.WriteLine("Wrote file {0}", outputStream.Name);
      }

      static CFileAttachment GetFileAttachmentToAdd(string path)
      {
         CFileAttachment attachFile = new CFileAttachment();
      
         //Get the file
         FileStream inputStream = File.OpenRead(path);
         int nSizeBuff = (int)inputStream.Length;
         BinaryReader reader = new BinaryReader(inputStream);
      
         attachFile.mpFileData = new System.Byte[nSizeBuff];
         attachFile.mpFileData = reader.ReadBytes(nSizeBuff);
         attachFile.mstrFileName = inputStream.Name;
         inputStream.Close();
      
         return attachFile;
      }
   }
}
