using System;
using System.Collections.Generic;
using System.Text;
using System.Web.Services;
using System.Web.Services.Protocols;
using System.IO;

namespace AttachmentsTCM
{
   class AttachmentsTCM
   {
      static void Main(string[] args)
      {
         ttsoapcgi svr = null;
         long cookie = 0;

         try
         {
            svr = new ttsoapcgi();

            CProject[] projs = svr.getProjectList("SOAPUser", "myPassword");
            cookie = svr.ProjectLogon(projs[0], "SOAPUser", "myPassword");

            Console.WriteLine("Getting test case 30 for edit");
            CTestCase tc = svr.editTestCaseByRecordID(cookie, 30, true);

            // Get the next script order
            long scriptOrder = 1;
            if (tc.attachmentlist != null)
            {
               foreach (CFileAttachment atch in tc.attachmentlist)
               {
                  if (atch.mstrType == "Script")
                  {
                     scriptOrder++;
                     Console.WriteLine("* Script file attachment {0} found with script order {1}", atch.mstrFileName, atch.mscriptOrder);
                  }
               }
            }
            
            if (tc.sccFileList != null)
            {
               foreach (CSCCFileRecord scc in tc.sccFileList)
               {
                  if (scc.mstrType == "Script")
                  {
                     scriptOrder++;
                     Console.WriteLine("* Script scc file record {0} found with script order {1}", scc.mstrFileName, scc.mscriptOrder);
                  }
               }
            }
            
            // Add a new script file attachment
            CFileAttachment tcScript = GetFileAttachmentToAdd(@"C:\Users\horakj\SOAP\Files\logonFirefoxTest.qawscript");
            tcScript.mstrType = "Script";
            tcScript.mscriptOrder = scriptOrder;
            tcScript.mscriptOrderSpecified = true;

            // Add a standard file attachment
            CFileAttachment tcStd = GetFileAttachmentToAdd(@"C:\Users\horakj\SOAP\Files\ExpectedResult_LoginScreen.png");

            // Resize our file attachment array and add the new file attachments at the end
            if (tc.attachmentlist != null)
            {
               CFileAttachment[] lst = tc.attachmentlist;
               Array.Resize<CFileAttachment>(ref lst, lst.Length + 2);
               tc.attachmentlist = lst;
               tc.attachmentlist[tc.attachmentlist.Length - 2] = tcScript;
               tc.attachmentlist[tc.attachmentlist.Length - 1] = tcStd;
            }
            else
               tc.attachmentlist = new CFileAttachment[] { tcScript, tcStd };

            // Add a new script SCC file record
            scriptOrder++;
            CSCCFileRecord tcSccScript = new CSCCFileRecord();
            tcSccScript.mstrFileName = "Project_Mainline::Project_Mainline/1.0.x/logonIETest.qawscript";
            tcSccScript.mstrType = "Script";
            tcSccScript.mscriptOrder = scriptOrder;
            tcSccScript.mscriptOrderSpecified = true;

            // Resize our SCC file array and add the new SCC record at the end
            if (tc.sccFileList != null)
            {
               CSCCFileRecord[] sccLst = tc.sccFileList;
               Array.Resize<CSCCFileRecord>(ref sccLst, sccLst.Length + 1);
               tc.sccFileList = sccLst;
               tc.sccFileList[tc.sccFileList.Length - 1] = tcSccScript;
            }
            else
               tc.sccFileList = new CSCCFileRecord[] { tcSccScript };

            Console.WriteLine("Saving test case with additional attachments");
            // Save the test case
            svr.saveTestCase(cookie, tc);

            Console.WriteLine("Generating a test run from the test case");
            // Generate a test run and edit the resulting run
            CItemToTrack[] testRunID = svr.generateTestRuns(cookie, tc.recordid, null, null);

            Console.WriteLine("Getting generated test run for edit");
            CTestRun tr = svr.editTestRunByRecordID(cookie, testRunID[0].recordid, true);

            // Add a run result attachment
            CFileAttachment trRunResult = GetFileAttachmentToAdd(@"C:\Users\horakj\SOAP\Files\ActualResult_LoginScreen.png");
            trRunResult.mstrType = "Run Result";

            // Resize the file attachment list and add the new attachment at the end
            if (tr.attachmentlist != null)
            {
               CFileAttachment[] lst = tr.attachmentlist;
               Array.Resize<CFileAttachment>(ref lst, lst.Length + 1);
               tr.attachmentlist = lst;
               tr.attachmentlist[tr.attachmentlist.Length - 1] = trRunResult;
            }
            else
               tr.attachmentlist = new CFileAttachment[] { trRunResult };

            Console.WriteLine("Added run result file attachment and saving test run");
            svr.saveTestRun(cookie, tr);
         }
         catch (SoapException e)
         {
            Console.WriteLine("Soap fault: {0} {1}", e.Detail.InnerText, e.Message.ToString());
         }
         catch (Exception e)
         {
            Console.WriteLine(e.ToString());
         }
         finally
         {
            try
            {
               if (svr != null)
               {
                  svr.DatabaseLogoff(cookie);
                  Console.WriteLine("Logged Off");
               }
            }
            catch (SoapException e)
            {
               Console.WriteLine("Soap fault: {0} {1}", e.Detail.InnerText, e.Message.ToString());
            }
            catch (Exception e)
            {
               Console.WriteLine(e.ToString());
            }
         }
      }

      static CFileAttachment GetFileAttachmentToAdd(string path)
      {
         CFileAttachment attachFile = new CFileAttachment();
      
         //Get the file
         FileStream inputStream = File.OpenRead(path);
         int nSizeBuff = (int)inputStream.Length;
         BinaryReader reader = new BinaryReader(inputStream);
      
         attachFile.mpFileData = new System.Byte[nSizeBuff];
         attachFile.mpFileData = reader.ReadBytes(nSizeBuff);
         attachFile.mstrFileName = inputStream.Name;
         inputStream.Close();
      
         return attachFile;
      }
   }
}
