﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Web.Services;
using System.Web.Services.Protocols;
using System.IO;
using System.Diagnostics;

namespace DocumentManagement
{
   class DocumentManagement
   {
      static void Main(string[] args)
      {
         // This will stream all output to the event log
         StringWriter log_output = null;
         try
         {
            log_output = new StringWriter();
         }
         catch (IOException e)
         {
            Console.WriteLine("{0} - Cannot create log output.", e.Message);
         }

         Console.SetOut(log_output);

         ttsoapcgi svr = null;
         long cookie = 0;

         try
         {
            svr = new ttsoapcgi();

            CProject[] projs = svr.getProjectList("SOAPUser", "myPassword");
            cookie = svr.ProjectLogon(projs[0], "SOAPUser", "myPassword");

            long id = long.Parse(Environment.GetEnvironmentVariable("RECORDID"));

            // First, take a snapshot of this document
            CSnapshotInfo snapShot = new CSnapshotInfo();
            snapShot.label = DateTime.Now.ToString();
            snapShot.comment = "Automatic snapshot from SOAP when moved into this state";
            svr.createSnapshot(cookie, id, snapShot);

            // Process all the requirements in the document. Generate test case for each functional requirement
            long[] reqIDs = svr.getRequirementIDsForDocument(cookie, id);
            foreach (long reqID in reqIDs)
            {
               CRequirement req = svr.getRequirementByRecordID(cookie, reqID, false);
               if (req.type == "Functional Requirement")
               {
                  // Generate a test case from this requirement
                  CTestCase tc = new CTestCase();
                  tc.summary = req.summary;
                  tc.type = "Requirements";
                  tc.steps = "#Generated from " + Environment.GetEnvironmentVariable("TTSTUDIOURL") + " with SOAP";
            
                  // Assign the test case
                  CEvent assignEvt = new CEvent();
                  assignEvt.name = "Assign";
                  assignEvt.user = "User, SOAP";
                  assignEvt.date = System.DateTime.Now;
                  assignEvt.notes = "A new functional requirement has been approved in document " +
                     Environment.GetEnvironmentVariable("RDNAM");
            
                  // Get the QA Owner from the Requirement custom field, assign the test case to that user
                  CDropdownField ownerFld = null;
                  for (int i = 0; i < req.customFieldList.Length; i++)
                  {
                     if (req.customFieldList[i].name == "QA Owner")
                     {
                        ownerFld = (CDropdownField)req.customFieldList[i];
                        break;
                     }
                  }
                  
                  if (ownerFld != null)
                     assignEvt.assigntolist = new string[] { ownerFld.value };
                  tc.eventlist = new CEvent[] { assignEvt };
            
                  // Add the generated test case
                  tc.recordid = svr.addTestCase(cookie, tc);
            
                  // Create linked item for the requirement
                  CLinkedItem reqParent = new CLinkedItem();
                  reqParent.tablename = "Requirement";
                  reqParent.entityID = reqID;
            
                  // Create linked item for the test case
                  CLinkedItem tcLinkItem = new CLinkedItem();
                  tcLinkItem.entityID = tc.recordid;
                  tcLinkItem.tablename = "Test Case";
                  tcLinkItem.issuspect = true;
                  tcLinkItem.issuspectSpecified = true;
            
                  // Add a Requirement Tested By link
                  CLink link = new CLink();
                  link.linkDefinitionName = "Requirement Tested By";
                  link.childList = new CLinkedItem[1] { tcLinkItem };
                  link.linkparent = reqParent;
                  svr.addLink(cookie, link);
               }
            }
         }
         catch (SoapException e)
         {
            Console.WriteLine("Soap fault: {0} {1}", e.Detail.InnerText, e.Message.ToString());
         }
         catch (Exception e)
         {
            Console.WriteLine(e.ToString());
         }
         finally
         {
            try
            {
               if (svr != null)
               {
                  svr.DatabaseLogoff(cookie);
                  Console.WriteLine("Logged Off");
               }
            }
            catch (SoapException e)
            {
               Console.WriteLine("Soap fault: {0} {1}", e.Detail.InnerText, e.Message.ToString());
            }
            catch (Exception e)
            {
               Console.WriteLine(e.ToString());
            }
         }

         log_output.Close();

         string sOutput = log_output.ToString();
         string sSource = "TestTrack Document Management Demo";

         if (sOutput.Length > 0)
         {
            if (!EventLog.SourceExists(sSource))
               EventLog.CreateEventSource(sSource, "Application");

            EventLog.WriteEntry(sSource, log_output.ToString());
         }
      }
   }
}
