﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Web.Services;
using System.Web.Services.Protocols;
using System.Net.Mail;
using System.Net.Mime;
using System.IO;

namespace EmailReport
{
   class EmailReport
   {
      static void Main(string[] args)
      {
         ttsoapcgi svr = null;
         long cookie = 0;

         try
         {
            svr = new ttsoapcgi();

            CProject[] projs = svr.getProjectList("SOAPUser", "myPassword");
            cookie = svr.ProjectLogon(projs[0], "SOAPUser", "myPassword");

            // Run the distribution report
            Console.WriteLine("Run Distribution report");
            CReportRunResults results = svr.getReportRunResultsByRecordID(cookie, 1);
            
            // Initialize the mail message
            MailAddress fromAddr = new MailAddress("TestTrack@seapine.com", "TestTrack Report Mailer");
            MailAddress toAddr = new MailAddress("TestTrack.Demo@gmail.com", "Jeff Horak");
            MailMessage mail = new MailMessage(fromAddr, toAddr);
            mail.Subject = "Requirement Status Report";

            // Search for the HTML file which will be the email body
            int imgNum = 1;
            string body = null;
            List<LinkedResource> imgs = new List<LinkedResource>();
            foreach (CFile f in results.mFileList)
            {
               if (f.mstrFileName.EndsWith(".html"))
               {
                  Console.WriteLine("Processing {0}", f.mstrFileName);
                  body = System.Text.ASCIIEncoding.ASCII.GetString(f.mpFileData);
                  break;
               }
            }

            // Get each image or resource file
            foreach (CFile f in results.mFileList)
            {
               if (!f.mstrFileName.EndsWith(".html"))
               {
                  Console.WriteLine("Processing {0}", f.mstrFileName);
                  MemoryStream stream = new MemoryStream(f.mpFileData);
                  LinkedResource img = new LinkedResource(stream, "image/png");
                  img.ContentId = "image" + imgNum++;
                  imgs.Add(img);
            
                  // Replace the img in the HTML with the content ID
                  body = body.Replace(f.mstrFileName, "cid:" + img.ContentId);
               }
            }

            // Create the HTML alt view and add linked resources
            AlternateView altView = AlternateView.CreateAlternateViewFromString(body, null, System.Net.Mime.MediaTypeNames.Text.Html);
            foreach (LinkedResource resource in imgs)
               altView.LinkedResources.Add(resource);
            mail.AlternateViews.Add(altView);
            
            // Send the email
            SmtpClient client = new SmtpClient("lists.seapine.com", 25);
            try
            {
               client.Send(mail);
            }
            catch (Exception ex)
            {
               Console.WriteLine("Exception caught while sending mail: {0}", ex.ToString());
            }
         }
         catch (SoapException e)
         {
            Console.WriteLine("Soap fault: {0} {1}", e.Detail.InnerText, e.Message.ToString());
         }
         catch (Exception e)
         {
            Console.WriteLine(e.ToString());
         }
         finally
         {
            try
            {
               if (svr != null)
               {
                  svr.DatabaseLogoff(cookie);
                  Console.WriteLine("Logged Off");
               }
            }
            catch (SoapException e)
            {
               Console.WriteLine("Soap fault: {0} {1}", e.Detail.InnerText, e.Message.ToString());
            }
            catch (Exception e)
            {
               Console.WriteLine(e.ToString());
            }
         }
      }
   }
}
